@@ -1,49 +1,19 @@
 import vector_utils
 import numpy
 import svg.path
-# import xml.etree.ElementTree as ET
-import lxml.etree as ET
+import xml.etree.ElementTree as ET
 import os
 import copy
-import wrap_scour
-import warnings
 
 '''
 Compress svg figure through a variety of methods
-Written July 22, 2014
-Author: Gen Del Raye
 '''
 
 class Compress:
-    '''
-    Read in the paths in a svg file and compress selected polygons, then output the compressed
-    svg to file. The general workflow should follow:
-        comp_instance = compress(filename)                # Read svg
-        comp_instance.find_paths()                        # Extract path coordinates
-        comp_instance.linearize_paths(curve_fidelity)     # Convert paths into polygons
-        comp_instance.compress_by_x(*args, **kwargs)      # Compress polygons ( where 'x' = 'merging',
-                                                          # 'deletion', or 'simplification').
-        comp_instance.write(outputfile = outputfile)      # Output compressed file
-        comp_instance.report()                            # Report on compression stats
-        
-    Most of this functionality can also be accessed through a convenience function 'compress_by_method()'
-    See usage examples in test()
-    '''
     def __init__(self, svg_file):
-        '''
-        Read svg from path 'svg_file' (e.g. 'test/test_vector.svg')
-        '''
         self.filename, self.extension = svg_file.split('.') # Get filename and extension
         assert self.extension == 'svg', 'File must be an svg'
-        try:
-            self.figure_data = ET.parse(svg_file) # Parse svg data as xml
-        except ET.XMLSyntaxError, e:
-            '''
-            Large svgs may trigger lxml's 'excessive depth in document' error
-            '''
-            warnings.warn('lxml error: %s - Trying huge xml parser' %(e.message))
-            huge_parser = ET.XMLParser(huge_tree = True)
-            self.figure_data = ET.parse(svg_file, parser = huge_parser)
+        self.figure_data = ET.parse(svg_file) # Parse svg data as xml
         self.root = self.figure_data.getroot() # Root object in svg
         self.reportString = '----------- REPORT -----------' # String used to generate reports
         self.initial_kb = self._get_kb(svg_file) # Get the uncompressed file size in KBytes
@@ -55,8 +25,7 @@ class Compress:
         tag_prefix = self._get_tag_prefix()
         self.path_nodes = self.root.findall('.//%spath' %(tag_prefix))
         self.path_parents = self.root.findall('.//%spath/..' %(tag_prefix))
-        self.paths = list()
-        self.paths = [svg.path.parse_path(p.attrib.get('d', 'M 0,0 z')) for p in self.path_nodes]
+        self.paths = [svg.path.parse_path(p.attrib['d']) for p in self.path_nodes]
         
     def linearize_paths(self, curve_fidelity = 10):
         '''
@@ -70,13 +39,6 @@ class Compress:
     def compress_by_deletion(self, criteria, threshold, search_within = None):
         '''
         Remove paths from svg that fail the given criteria
-        Inputs:
-            criteria(str) - criteria by which to accept or reject polygons. If 'bboxarea', 
-              the criteria is the area of the bounding box of a polygon, while if 'circumference',
-              the criteria is the circumference of the polygon.
-            threshold(scalar) - threshold value (e.g. of area) below which polygons are marked for
-              deletion.
-            search_within(iterable) - array of indices within which to search
         '''
         deletion_index = self.search_polygons(criteria, threshold, search_within)
         self._delete_paths(deletion_index)
@@ -89,11 +51,6 @@ class Compress:
         '''
         Find indices of polygons that fail given criteria
         Inputs:
-            criteria(string) - criteria by which to accept or reject polygons. If 'bboxarea', 
-              the criteria is the area of the bounding box of a polygon, while if 'circumference',
-              the criteria is the circumference of the polygon.
-            threshold(scalar) - threshold value (e.g. of area) below which polygons are marked for
-              deletion.
             search_within(ndarray) - array of indices within which to search
         '''
         polygons = self._coord2polygon(self.linear_coords)
@@ -143,7 +100,7 @@ class Compress:
     
     def _coord2polygon(self, coords):
         '''
-        Convert a list or array of coordinates in coords into a polygon object
+        Convert a list or array of coordinates into a polygon object
         '''
         return [vector_utils.Polygon(coord) for coord in coords]
     
@@ -151,8 +108,6 @@ class Compress:
         '''
         Simplify polygons in the svg using the Ramer-Douglas-Peucker algorithm
         Inputs:
-            epsilon(scalar) - scalar metric of how far the simplified line is allowed to 
-              stray from the original.
             search_within(ndarray) - indices of paths to simplify
         '''
         replace_coords = numpy.array(self.linear_coords)
@@ -167,8 +122,7 @@ class Compress:
     
     def overlap_list(self, poly, polyarray, search_index):
         '''
-        Return list of polygons in a polyarray that intersect with poly
-        Written to be used as port of recursive_intersect()
+        Return list of polygons in an array that intersect with poly
         '''
         overlapList = list()
         indexList = list()
@@ -180,11 +134,6 @@ class Compress:
     
     def recursive_intersect(self, poly, pArray, intersectList = [], intersectIndex = [], search_index = [], 
                             nterminate = numpy.Inf, bufferDistance = None):
-        '''
-        Find all polygons that either intersect with, or intersect with polygons that intersect with,
-        a given polygon.
-        Written to be used as part of compress_by_merging
-        '''
         if bufferDistance is not None:
             poly.buffer(bufferDistance)
         childeren, childeren_index = self.overlap_list(poly, pArray, search_index)
@@ -203,34 +152,18 @@ class Compress:
     
     def _merge(self, polygon, intersect):
         '''
-        Merge polygons in intersect with polygon using the cascading union
+        Append polygon by cascading union
         '''
         polygon.cascading_union(intersect[1:])
         
     def _hull(self, polygon, intersect):
         '''
-        Merge polygons in intersect with polygon using the minimum convex hull
+        Append polygon by convex hull
         '''
         polygon.hull(intersect[1:])
     
-    def compress_by_merging(self, search_within = None, epsilon = 1, nterminate = numpy.Inf, group_by_color = True, 
-                            bufferDistance = None, operation_key = 'merge'):
-        '''
-        Compress svg by merging overlapping or neighboring polygons
-        Inputs:
-            search_within(iterable) - a list or 1d array of indices for which to search in self.path_nodes.
-              Optional, defaults to None.
-            epsilon(scalar) - metric of how far the simplified line is allowed to stray from the original.
-              Optional, defaults to 1
-            nterminate(scalar) - upper limit to the number of recursions that should be performed while
-              searching for overlapping or neighboring polygons. Optional, defaults to numpy.Inf
-            group_by_color(bool) - if False, merge neighboring or overlapping polygons of separate colors.
-              Optional, defaults to True.
-            bufferDistance(scalar) - limit of distance by which two polygons can be separated and still be
-              merged. Defaults to None.
-            operation_key(str) - the method by which merged polygons should be constructed. 'merge' uses
-              shapely.cascading_union, 'hull' constructs a minimum convex hull.
-        '''
+    def compress_by_merging(self, search_within = None, epsilon = 1, nterminate = numpy.Inf, group_by_color = True, bufferDistance = None,
+                            operation_key = 'merge'):
         get_operation = {'merge': self._merge,
                          'hull': self._hull}
         operation = get_operation[operation_key]
@@ -238,8 +171,7 @@ class Compress:
         n_deleted = 0 # Reporting variable for number of polygons unified
         if group_by_color:
             nullstyle = 'fill:#zzzzzz;fill-opacity:-1' # Some paths (e.g. clipping paths) don't have styles
-            styles = [vector_utils.parse_style(p.attrib.get('style', nullstyle)) for p in self.path_nodes]
-            self.colors = numpy.array([style.get('fill', '#zzzzzz') for style in styles])
+            self.colors = numpy.array([vector_utils.parse_style(p.attrib.get('style', nullstyle))['fill'] for p in self.path_nodes])
             color_str = numpy.unique(self.colors)
             color_groupings = [numpy.where(self.colors == c)[0] for c in color_str]
         for color in color_groupings:
@@ -248,7 +180,6 @@ class Compress:
             search_space = numpy.intersect1d(search_within, color, assume_unique = True)
             search_coords = numpy.array(self.linear_coords)[search_space]
             search_index = list(search_space)
-            # TODO: Make the polygon search more efficient by using rtrees
             polygons = self._coord2polygon(search_coords)
             while len(polygons)>0:
                 p1 = polygons.pop()
@@ -272,17 +203,13 @@ class Compress:
         
         
     def _replace_paths(self, replacement_index, replacement_str):
-        '''
-        Replace path coordinates in an svg with a string formatted in svg 
-        format (e.g. 'M 50,50 50,100 100,100 100,50 50,50 z')
-        '''
         for index, path in zip(replacement_index, replacement_str):
 #             self.path_nodes[index].set('d', 'M 50,50 50,100 100,100 100,50 50,50 z') # debug
             self.path_nodes[index].set('d', path)
             
     def write(self, outputfile = None):
         '''
-        Write compressed svg to file. Not necessary if self.optimize() has been used.
+        Write compressed svg to file
         Inputs:
             outputfile - filename to write to (e.g. 'output.svg'). If not given,
                 will write to 'originalfilename_compressed.svg'
@@ -293,7 +220,6 @@ class Compress:
         final_kb = self._get_kb(outputfile) # Get the compressed size in KBytes
         self._append_report('Compressed %sKB to %sKB' %(self.initial_kb, final_kb))
         self._append_report('Saved to %s' %(outputfile))
-        return outputfile
     
     def report(self):
         '''
@@ -302,9 +228,6 @@ class Compress:
         print self.reportString
 
     def _append_report(self, new_text):
-        '''
-        Add a new line to the report
-        '''
         self.reportString = '\n'.join((self.reportString, new_text))
     
     def _get_tag_prefix(self):
@@ -313,75 +236,25 @@ class Compress:
         e.g. if the root tag is '{http://www.w3.org/2000/svg}svg', all other tags 
         in the xml are probably prefixed by '{http://www.w3.org/2000/svg}'
         '''
-        # TODO: Fix this hack
         prefix_endindex = self.root.tag.rindex('}')
         return self.root.tag[:(prefix_endindex + 1)]
     
     def _get_kb(self, filename):
-        '''
-        Return size of the file 'filename' in KB.
-        '''
         return os.path.getsize(filename)/1000.0
-    
-    def optimize(self, inputfile, outputfile = None):
-        '''
-        Use package 'Scour' to optimize the compressed svg (e.g. remove
-        unused nodes).
-        '''
-        if outputfile == None:
-            outputfile = '%s_opt.svg' %(self.filename)
-#            Defaults:
-#                 {'strip_ids': False, 'shorten_ids': False, 'simple_colors': True, 
-#                  'strip_comments': False, 'remove_metadata': False, 'outfilename': None, 
-#                  'group_create': False, 'protect_ids_noninkscape': False, 'indent_type': 'space', 
-#                  'keep_editor_data': False, 'shorten_ids_prefix': '', 'renderer_workaround': True, 
-#                  'style_to_xml': True, 'protect_ids_prefix': None, 'enable_viewboxing': False, 
-#                  'digits': 5, 'embed_rasters': True, 'infilename': None, 'strip_xml_prolog': False, 
-#                  'group_collapse': True, 'quiet': False, 'protect_ids_list': None}
-        wrap_scour.optimize(self.root, inputfile, outputfile, strip_ids = True, shorten_ids = True, 
-                            simple_colors = True, strip_comments = True, remove_metadata = True, 
-                            group_create = True, protect_ids_noninkscape = False,
-                            indent_type = None, keep_editor_data = False, renderer_workaround = False, 
-                            style_to_xml = True, digits = 5, embed_rasters = True, strip_xml_prolog = True, 
-                            group_collapse = True)
-        final_kb = self._get_kb(outputfile) # Get the compressed size in KBytes
-        self._append_report('Compressed %sKB to %sKB' %(self.initial_kb, final_kb))
-        self._append_report('Saved to %s' %(outputfile))
 
 def compress_by_method(filename, compression_type, curve_fidelity = 10, outputfile = None,
-                       pre_select = False, selection_tuple = ('', ''), optimize = True, **kwargs):
+                       pre_select = False, selection_tuple = ('', ''), **kwargs):
     '''
-    Convenience function for 1-step compression of svg files.
-    Inputs:
-        filename(str) - path to svg file to compress, e.g. 'test_vector.svg'
-        compression_type(str) - how to carry out the compression. Options are 'delete' 
-          (remove polygons by size), 'simplify' (simplify polygons), and 'merge' (merge 
-          neighboring or overlapping polygons).
-        curve_fidelity(int) - all polygons affected by compression (e.g. those that are 
-          simplified) need to first be linearized so as to convert smooth curves into a set
-          of discrete coordinates. curve_fidelity sets the number of coordinates to use to 
-          interpolate a curve. The larger the number, the more computation the code will need 
-          to perform. Optional, defaults to 10.
-        outputfile(str) - path to output svg. Optional, defaults to 'originalname_compressed.svg'
-        pre_select(bool) - if True, will perform the compression only on a subset of the 
-          polygons in the svg that are below a certain size threshold. Optional, defaults to False.
-        selection_tuple(tuple) - if pre_select == True, the selection tuple ('criteria', 
-          threshold) determines the criteria for selection ('bboxarea' for the area of the 
-          bounding box of a polygon, or 'circumference' for its circumference) and the threshold 
-          size. Optional, defaults to ('', '')
-        optimize(bool) - if True, will use the package Scour to optimize compressed svg (e.g.
-          remove redundant nodes, id attributes). Saves optimized svg to 'outputfile_opt.svg'
-        **kwargs - depending on the compression type and the pre-selection criteria, additional 
-          key word arguments may be required. See compress_by_merging, compress_by_deletion, and 
-          compress_by_simplification in class Compress for the valid keyword arguments when 
-          compression_type = 'merge', 'delete', and 'simplify' respectively.
-    Outputs:
-        Compressed svg in the directory given by outputfile or in the code directory with the 
-          name 'originalname_compressed.svg'.
-        A report string outputted to the console will also state some general indicators about 
-          the success of the compression, the path of the outputted svg, and the initial and 
-          compressed file sizes.
+    Convenience method for one-pass compression
+    See compress_by_merging, compress_by_deletion, and compress_by_simplification in Compress
+    for kwargs.
     '''
+    def compression_options(compress_instance, option_key, **kwargs):
+        compression_options = {'merge': compress_instance.compress_by_merging,
+                               'delete': compress_instance.compress_by_deletion,
+                               'simplify': compress_instance.compress_by_simplification}
+        compression_options[option_key](**kwargs)
+
     test_compression = Compress(filename)
     test_compression.find_paths()
     test_compression.linearize_paths(curve_fidelity = curve_fidelity)
@@ -389,17 +262,8 @@ def compress_by_method(filename, compression_type, curve_fidelity = 10, outputfi
     if pre_select:
         criteria, threshold = selection_tuple
         pre_selection = test_compression.search_polygons(criteria, threshold)
-
-    compression_options = {'merge': test_compression.compress_by_merging,
-                           'delete': test_compression.compress_by_deletion,
-                           'simplify': test_compression.compress_by_simplification}
-    compress_by_x = compression_options[compression_type]
-    compress_by_x(search_within = pre_selection, **kwargs)
-    inputfile = test_compression.write(outputfile)
-    if optimize:
-        outfilename, extension = outputfile.split('.') # Get filename and extension
-        opti_output = '%s_opt.%s' %(outfilename, extension)
-        test_compression.optimize(inputfile, opti_output)
+    compression_options(test_compression, compression_type, search_within = pre_selection, **kwargs)
+    test_compression.write(outputfile = outputfile)
     test_compression.report()
 
 def test(filename, thresholds = (5, 2), epsilon = 0.5, curve_fidelity = 10, 
@@ -441,6 +305,4 @@ if __name__ == '__main__':
     # Demonstrate all functions with pre-prepared test vector
     test(filename = 'test/test_vector.svg', thresholds = (300, 150), epsilon = 5, buffer_distance = 10)
     # Demonstrate all functions with sample map
-    test(filename = 'test/map_test.svg', thresholds = (6, 3), epsilon = 30, buffer_distance = 20)
-    # Demonstrate with a plot constructed in matplotlib
-    test(filename = 'test/matplotlib_test.svg', thresholds = (1000, 100), epsilon = 2, buffer_distance = 200)
+    test(filename = 'test/map_test.svg', thresholds = (6, 3), epsilon = 30, buffer_distance = 50)
